/*
 * Test file access.
 *
 * Copyright (C) 2020, 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>
#include <l4/sys/err.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <fsclient/client.h>
#include <systypes/fcntl.h>



int main(int argc, char *argv[])
{
  if (argc < 2)
  {
    printf("Need directory names.\n");
    return 1;
  }

  for (int i = 1; i < argc; i++)
  {
    char *dirname = argv[i];

    printf("Opening %s...\n", dirname);

    file_t *reader = client_opendir(dirname);

    if (!client_opened(reader))
    {
      client_close(reader);
      printf("Could not read from directory: %s\n", dirname);
      continue;
    }

    printf("  Reading...\n");

    struct dirent *dirent;

    while ((dirent = client_readdir(reader)) != NULL)
    {
      char *filename = dirent->d_name;
      if (!strlen(filename) || (filename[0] == '.'))
        continue;

      char path[strlen(filename) + strlen(dirname) + 2];

      sprintf(path, "%s/%s", dirname, filename);

      printf("  Attempt to read %s...\n", filename);

      file_t *file = client_open(path, O_RDONLY);

      if (!client_opened(file))
        printf("    Could not open file: %s\n", filename);
      else
      {
        char buffer[32];
        offset_t nread = client_read(file, buffer, 32);

        fputs("    Start of file:\n----\n", stdout);
        fwrite(buffer, sizeof(char), nread, stdout);
        fputs("\n----\n", stdout);
      }

      client_close(file);
      free(dirent);
    }

    client_close(reader);
    printf("Closing directory %s.\n\n", dirname);
  }

  printf("End of test.\n");
  return 0;
}

// vim: tabstop=2 expandtab shiftwidth=2
